'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _findCacheDir = require('find-cache-dir');

var _findCacheDir2 = _interopRequireDefault(_findCacheDir);

var _nodeLogger = require('@storybook/node-logger');

var _webpack = require('./config/defaults/webpack.config');

var _babel_config = require('./babel_config');

var _babel_config2 = _interopRequireDefault(_babel_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable global-require, import/no-dynamic-require */
var noopWrapper = function noopWrapper(config) {
  return config;
};

function setBabelLoaderOptions(config, configDir, defaultBabelConfig, wrapBabelConfig) {
  var jsRule = config.module.rules[0];

  var babelConfig = wrapBabelConfig((0, _babel_config2.default)(configDir, defaultBabelConfig));

  jsRule.query = (0, _extends3.default)({
    // This is a feature of `babel-loader` for webpack (not Babel itself).
    // It enables a cache directory for faster-rebuilds
    // `find-cache-dir` will create the cache directory under the node_modules directory.
    cacheDirectory: (0, _findCacheDir2.default)({ name: 'react-storybook' })
  }, babelConfig);
}

function setConfigJsToEntry(config, configDir) {
  // Check whether a config.js file exists inside the storybook
  // config directory and throw an error if it's not.
  var storybookConfigPath = _path2.default.resolve(configDir, 'config.js');
  if (!_fs2.default.existsSync(storybookConfigPath)) {
    throw new Error('=> Create a storybook config file in "' + configDir + '/config.js".');
  }

  config.entry.preview.push(require.resolve(storybookConfigPath));
}

function setAddonsJsToEntry(config, configDir) {
  // Check whether addons.js file exists inside the storybook.
  var storybookCustomAddonsPath = _path2.default.resolve(configDir, 'addons.js');
  if (_fs2.default.existsSync(storybookCustomAddonsPath)) {
    _nodeLogger.logger.info('=> Loading custom addons config.');
    config.entry.manager.unshift(storybookCustomAddonsPath);
  }
}

function mergeConfigs(config, customConfig) {
  return (0, _extends3.default)({}, customConfig, config, {
    // Override with custom devtool if provided
    devtool: customConfig.devtool || config.devtool,
    // We need to use our and custom plugins.
    plugins: [].concat((0, _toConsumableArray3.default)(config.plugins), (0, _toConsumableArray3.default)(customConfig.plugins || [])),
    module: (0, _extends3.default)({}, config.module, customConfig.module, {
      rules: [].concat((0, _toConsumableArray3.default)(config.module.rules), (0, _toConsumableArray3.default)(customConfig.module && customConfig.module.rules || []))
    }),
    resolve: (0, _extends3.default)({}, config.resolve, customConfig.resolve, {
      alias: (0, _extends3.default)({}, config.alias, customConfig.resolve && customConfig.resolve.alias)
    })
  });
}

function informAboutCustomConfig(defaultConfigName) {
  if (!defaultConfigName) {
    _nodeLogger.logger.info('=> Using default webpack setup.');
    return;
  }

  _nodeLogger.logger.info('=> Using default webpack setup based on "' + defaultConfigName + '".');
}

// `baseConfig` is a webpack configuration bundled with storybook.
// Storybook will look in the `configDir` directory
// (inside working directory) if a config path is not provided.

exports.default = function (options) {
  return function (configType, baseConfig, configDir) {
    var defaultConfigName = options.defaultConfigName,
        defaultBabelConfig = options.defaultBabelConfig,
        _options$wrapBabelCon = options.wrapBabelConfig,
        wrapBabelConfig = _options$wrapBabelCon === undefined ? noopWrapper : _options$wrapBabelCon,
        _options$wrapInitialC = options.wrapInitialConfig,
        wrapInitialConfig = _options$wrapInitialC === undefined ? noopWrapper : _options$wrapInitialC,
        _options$wrapBasicCon = options.wrapBasicConfig,
        wrapBasicConfig = _options$wrapBasicCon === undefined ? noopWrapper : _options$wrapBasicCon,
        _options$wrapDefaultC = options.wrapDefaultConfig,
        wrapDefaultConfig = _options$wrapDefaultC === undefined ? noopWrapper : _options$wrapDefaultC;


    var config = wrapInitialConfig(baseConfig, configDir);

    setBabelLoaderOptions(config, configDir, defaultBabelConfig, wrapBabelConfig);
    setConfigJsToEntry(config, configDir);
    setAddonsJsToEntry(config, configDir);

    var defaultConfig = wrapDefaultConfig((0, _webpack.createDefaultWebpackConfig)(config));

    // Check whether user has a custom webpack config file and
    // return the (extended) base configuration if it's not available.
    var customConfigPath = _path2.default.resolve(configDir, 'webpack.config.js');

    if (!_fs2.default.existsSync(customConfigPath)) {
      informAboutCustomConfig(defaultConfigName);
      return defaultConfig;
    }

    var customConfig = require(customConfigPath);

    if (typeof customConfig === 'function') {
      _nodeLogger.logger.info('=> Loading custom webpack config (full-control mode).');
      return customConfig(wrapBasicConfig(config), configType, defaultConfig);
    }

    _nodeLogger.logger.info('=> Loading custom webpack config (extending mode).');

    return mergeConfigs(config, customConfig);
  };
};